/*
	:								DataFlash.c
	:						  DataFlash   ATMEL
	:							 
	:							08.06.2008
	:						09.06.2008
	:		Atmel AVR   SPI
	:							IAR Embedded Workbench
*/


#include "header.h"


#ifdef DATAFLASH_H

/* ---  --- */
// 
void dataflashInit ( void )
{
#ifdef DATAFLASH_RESET_DDR
	DATAFLASH_CONFIG_RESET;
#endif
#ifdef DATAFLASH_RESET_PIN
	DATAFLASH_RELEASE_RESET;
#endif
	DATAFLASH_CONFIG_CS;
	DATAFLASH_RELEASE_CS;
}

//   
unsigned char dataflashReadStatus ( void )
{
	unsigned char Status;

	DATAFLASH_ASSERT_CS;	//CS
	spiSend(DATAFLASH_CMD_RDSR);	//
	Status = spiTransfer(DATAFLASH_BLANKBYTE); // 
	DATAFLASH_RELEASE_CS;
	return Status;
}

//   
void dataflashChipErase ( void )
{
	DATAFLASH_ASSERT_CS;
	spiSend(0xC7);
	spiSend(0x94);
	spiSend(0x80);
	spiSend(0x9A);
	DATAFLASH_RELEASE_CS;
	__delay_cycles(CPU_clock_freq/1000000); // 1 
	DATAFLASH_ASSERT_CS;
	spiSend(DATAFLASH_CMD_RDSR);
	while (!(spiTransfer(DATAFLASH_BLANKBYTE) & DATAFLASH_BUSY));
	DATAFLASH_RELEASE_CS;
}

// ,     Addr
void dataflashSectorErase ( unsigned long Addr )
{
}

// ,     Addr
void dataflashBlockErase ( unsigned long Addr )
{
}

// ,     Addr
void dataflashPageErase ( unsigned long Addr )
{
}

//   BCnt    Data
void dataflashRead ( unsigned long Addr, unsigned int BCnt, unsigned char * Data )
{
	DATAFLASH_ASSERT_CS;
	spiSend(DATAFLASH_CMD_ARRDL);
	spiSend(Addr>>16);
	spiSend(Addr>>8);
	spiSend(Addr>>0);
	while (BCnt--)
		*Data++ = spiTransfer(DATAFLASH_BLANKBYTE);
	DATAFLASH_RELEASE_CS;
}

// BCnt    Data  
void dataflashPageProgram ( unsigned long Addr, unsigned int BCnt, unsigned char * Data )
{
	DATAFLASH_ASSERT_CS;
	spiSend(DATAFLASH_CMD_MMPPBUF1);
	spiSend(Addr>>16);
	spiSend(Addr>>8);
	spiSend(Addr>>0);
	while (BCnt--)
		spiSend(*Data++);
	DATAFLASH_RELEASE_CS;
	__delay_cycles(CPU_clock_freq/1000000); // 1 
	DATAFLASH_ASSERT_CS;
	spiSend(DATAFLASH_CMD_RDSR);
	while (!(spiTransfer(DATAFLASH_BLANKBYTE) & DATAFLASH_BUSY));
	DATAFLASH_RELEASE_CS;
}

#ifdef BUFFER_H
//       Buffer
void dataflashReadToCBuffer ( unsigned long Addr, unsigned long BCnt, cBuffer * Buffer )
{
	DATAFLASH_ASSERT_CS;
	spiSend(DATAFLASH_CMD_ARRDL);
	spiSend(Addr>>16);
	spiSend(Addr>>8);
	spiSend(Addr>>0);
	while(BCnt--)
		bufferAddToEnd(Buffer,spiTransfer(DATAFLASH_BLANKBYTE));
	DATAFLASH_RELEASE_CS;
}

//   
void dataflashPageUpdateFromCBuffer ( unsigned long Addr, unsigned int BCnt, cBuffer * Buffer )
{
	unsigned char Data;
	
	DATAFLASH_ASSERT_CS;
	spiSend(DATAFLASH_CMD_MMBUF1TR); //   1
	spiSend(Addr>>16);
	spiSend(Addr>>8);
	spiSend(Addr>>0);
	DATAFLASH_RELEASE_CS;
	__delay_cycles(CPU_clock_freq/1000000); // 1 
	DATAFLASH_ASSERT_CS;
	spiSend(DATAFLASH_CMD_RDSR);
	while (!(spiTransfer(DATAFLASH_BLANKBYTE) & DATAFLASH_BUSY)); //  
	DATAFLASH_RELEASE_CS;
	__delay_cycles(CPU_clock_freq/1000000); // 1 
	DATAFLASH_ASSERT_CS;
	spiSend(DATAFLASH_CMD_BUF1WR); //  1 
	spiSend(Addr>>16);
	spiSend(Addr>>8);
	spiSend(Addr>>0);
	while (BCnt--)
	{	
		bufferGetFromFront(Buffer,&Data,1);
		spiSend(Data);
	}
	DATAFLASH_RELEASE_CS;
	__delay_cycles(CPU_clock_freq/1000000); // 1 
	DATAFLASH_ASSERT_CS;
	spiSend(DATAFLASH_CMD_BUF1MMPPE); //    1
	spiSend(Addr>>16);
	spiSend(Addr>>8);
	spiSend(Addr>>0);
	DATAFLASH_RELEASE_CS;
	__delay_cycles(CPU_clock_freq/1000000); // 1 
	DATAFLASH_ASSERT_CS;
	spiSend(DATAFLASH_CMD_RDSR);
	while (!(spiTransfer(DATAFLASH_BLANKBYTE) & DATAFLASH_BUSY)); //  
	DATAFLASH_RELEASE_CS;
}
#endif

#endif
