/*
	:								DataFlash.h
	:						  DataFlash   ATMEL
	:							 
	:							08.06.2008
	:						08.06.2008
	:		Atmel AVR   SPI
	:							IAR Embedded Workbench
*/


#ifndef DATAFLASH_H
#define DATAFLASH_H

/* ---  --- */
#define DATAFLASH_BUSY									0x80

#define DATAFLASH_CMD_MMPGRD						0xD2	//main memory page read
#define DATAFLASH_CMD_ARRD							0xE8	//Continuous Array Read (Legacy Command)
#define DATAFLASH_CMD_ARRDL							0x03	//Continuous Array Read (Low Frequency)
#define DATAFLASH_CMD_ARRDH							0x0B	//Continuous Array Read (High Frequency)
#define DATAFLASH_CMD_BUF1RDL						0xD1	//Buffer 1 Read (Low Frequency)
#define DATAFLASH_CMD_BUF2RDL						0xD3	//Buffer 2 Read (Low Frequency)
#define DATAFLASH_CMD_BUF1RD						0xD4	//Buffer 1 Read
#define DATAFLASH_CMD_BUF2RD						0xD6	//Buffer 2 Read
#define DATAFLASH_CMD_BUF1WR						0x84	//Buffer 1 Write
#define DATAFLASH_CMD_BUF2WR						0x87	//Buffer 2 Write
#define DATAFLASH_CMD_BUF1MMPPE					0x83	//Buffer 1 to Main Memory Page Program with Built-in Erase
#define DATAFLASH_CMD_BUF2MMPPE					0x86	//Buffer 2 to Main Memory Page Program with Built-in Erase
#define DATAFLASH_CMD_BUF1MMPP					0x88	//Buffer 1 to Main Memory Page Program without Built-in Erase
#define DATAFLASH_CMD_BUF2MMPP					0x89	//Buffer 2 to Main Memory Page Program without Built-in Erase
#define DATAFLASH_CMD_PE								0x81	//Page Erase
#define DATAFLASH_CMD_BE								0x50	//Block Erase
#define DATAFLASH_CMD_SE								0x7C	//Sector Erase
#define DATAFLASH_CMD_MMPPBUF1					0x82	//Main Memory Page Program Through Buffer 1
#define DATAFLASH_CMD_MMPPBUF2					0x85	//Main Memory Page Program Through Buffer 2
#define DATAFLASH_CMD_MMBUF1TR					0x53	//Main Memory Page to Buffer 1 Transfer
#define DATAFLASH_CMD_MMBUF2TR					0x55	//Main Memory Page to Buffer 2 Transfer
#define DATAFLASH_CMD_MMBUF1CMP					0x60	//Main Memory Page to Buffer 1 Compare
#define DATAFLASH_CMD_MMBUF2CMP					0x61	//Main Memory Page to Buffer 2 Compare
#define DATAFLASH_CMD_PGRWBUF1					0x58	//Auto Page Rewrite through Buffer 1
#define DATAFLASH_CMD_PGRWBUF2					0x59	//Auto Page Rewrite through Buffer 1
#define DATAFLASH_CMD_DPWRDN						0xB9	//Deep Power-down
#define DATAFLASH_CMD_RDPWRDN						0xAB	//Resume from Deep Power-down
#define DATAFLASH_CMD_RDSR							0xD7	//Status Register Read
#define DATAFLASH_CMD_MDIDRD						0x9F	//Manufacturer and Device ID Read

#ifndef DATAFLASH_PAGESIZE
	#define	DATAFLASH_PAGESIZE						256
#endif

#ifndef DATAFLASH_BLANKBYTE
	#define DATAFLASH_BLANKBYTE					0xFF
#endif

#ifndef DATAFLASH_CS_PIN
	#error	  CS! (Pin CS not defined!)
#else
	#define DATAFLASH_ASSERT_CS		DATAFLASH_CS_PIN = 0
	#define DATAFLASH_RELEASE_CS		DATAFLASH_CS_PIN = 1
#endif

#ifndef DATAFLASH_CS_DDR
	#error	  CS! (Pin CS not defined!)
#else
	#define DATAFLASH_CONFIG_CS		DATAFLASH_CS_DDR = 1
#endif

#ifdef DATAFLASH_RESET_PIN
	#define DATAFLASH_ASSERT_RESET		DATAFLASH_RESET_PIN = 0
	#define DATAFLASH_RELEASE_RESET		DATAFLASH_RESET_PIN = 1
#endif

#ifdef DATAFLASH_RESET_DDR
	#define	DATAFLASH_CONFIG_RESET		DATAFLASH_RESET_DDR = 1
#endif


/* ---  --- */
extern void dataflashInit ( void );
extern void dataflashChipErase ( void );
extern void dataflashRead ( unsigned long Addr, unsigned int BCnt, unsigned char * Data );
extern void dataflashPageProgram ( unsigned long Addr, unsigned int BCnt, unsigned char * Data );
#ifdef BUFFER_H
extern void dataflashReadToCBuffer ( unsigned long Addr, unsigned long BCnt, cBuffer * Buffer );
extern void dataflashPageUpdateFromCBuffer ( unsigned long Addr, unsigned int BCnt, cBuffer * Buffer );
#endif

#endif
