/*------------------------------------------------------------------------------
	 buffer.c
	        
	: 30.08.2007
	: 22.05.2008
------------------------------------------------------------------------------*/


#include "header.h"


#ifdef BUFFER_H
/* ---  --- */
void bufferInit(cBuffer * Buffer, unsigned char * Start, unsigned int Size) // 
{
	CRITICAL_SECTION_START; //  
	Buffer->Dataptr = Start; //  
	Buffer->Size = Size; //  
	//    
	Buffer->Dataindex = 0;
	Buffer->Datalength = 0;
	CRITICAL_SECTION_END; //  
}

//   Data     
void  bufferGetFromFront(cBuffer * Buffer, unsigned char * Data, unsigned int ByteCount)
{
	unsigned int Counter;
	
	CRITICAL_SECTION_START; //  
	if(Buffer->Datalength>=ByteCount) //      
		for (Counter=0;Counter<ByteCount;Counter++)
		{
			Data[Counter] = Buffer->Dataptr[Buffer->Dataindex]; //   
			Buffer->Dataindex++; // 
			if (Buffer->Dataindex >= Buffer->Size) //   ,    
			{
				Buffer->Dataindex -= Buffer->Size;
			}
			Buffer->Datalength--; //    
		}
	CRITICAL_SECTION_END; //  
}

//      numbytes
void bufferDumpFromFront(cBuffer * Buffer, unsigned int Numbytes)
{
	CRITICAL_SECTION_START; //  
	if (Numbytes < Buffer->Datalength) //   ?
	{
		Buffer->Dataindex += Numbytes; //    numbytes
		if(Buffer->Dataindex >= Buffer->Size) //   ?
		{
			Buffer->Dataindex -= Buffer->Size; //  
		}
		Buffer->Datalength -= Numbytes; //    numbytes
	}
	else
	{
		Buffer->Datalength = 0; //    
	}
	CRITICAL_SECTION_END; //  
}

//   
unsigned char bufferGetAtIndex(cBuffer * Buffer, unsigned int Index)
{
	CRITICAL_SECTION_START; //  
	unsigned char Data = Buffer->Dataptr[(Buffer->Dataindex+Index)%(Buffer->Size)]; //   
	CRITICAL_SECTION_END; //  
	return Data;
}

//    
unsigned char bufferAddToEnd(cBuffer * Buffer, unsigned char Data)
{
	CRITICAL_SECTION_START; //  
	if (Buffer->Datalength < Buffer->Size) //,   
	{
		Buffer->Dataptr[(Buffer->Dataindex + Buffer->Datalength) % Buffer->Size] = Data; //   
		Buffer->Datalength++; //  
		CRITICAL_SECTION_END; //  
		return 1; //  
	}
	CRITICAL_SECTION_END; //  
	return 0; //  
}

//       Data  ByteCount
unsigned char bufferAddDataBlock(cBuffer * Buffer, unsigned char * Data, unsigned int ByteCount)
{
	unsigned int Counter;
	
	CRITICAL_SECTION_START; //  
	if (Buffer->Datalength + ByteCount <= Buffer->Size) //,       
	{
		for (Counter=0;Counter<ByteCount;Counter++) //     
		{
			Buffer->Dataptr[(Buffer->Dataindex + Buffer->Datalength) % Buffer->Size] = Data[Counter]; //   
			Buffer->Datalength++; //  
		}
		CRITICAL_SECTION_END; //  
		return 1; //  
	}
	CRITICAL_SECTION_END; //  
	return 0; //  
}

//,     
unsigned int bufferIsNotFull(cBuffer * Buffer)
{
	CRITICAL_SECTION_START; //  
	unsigned int Bytesleft = (Buffer->Size - Buffer->Datalength); //  
	CRITICAL_SECTION_END; //  
	return Bytesleft;
}

// 
void bufferFlush(cBuffer * Buffer)
{
	CRITICAL_SECTION_START; //  
	Buffer->Datalength = 0; // 
	CRITICAL_SECTION_END; //  
}

//    
void bufferChangeAtIndex(cBuffer * Buffer, unsigned int Index, unsigned char Data)
{
	CRITICAL_SECTION_START; //  
	Buffer->Dataptr[(Buffer->Dataindex+Index)%(Buffer->Size)] = Data;
	CRITICAL_SECTION_END; //  
}
#endif
