#include "header.h"


cBuffer RXBuffer,TXBuffer;
unsigned char Flags;
unsigned RXMode;
unsigned int tmrReceive;
unsigned int tmrSend;
unsigned char Counter1,Counter2;							//
unsigned int Crc16;														//    
unsigned int DataLength;


/* ---  --- */
__interrupt void USART_RXC_intr(void)
{
	bufferAddToEnd(&RXBuffer,UDR);
}

__interrupt void USART_TXC_intr(void)
{
	if (TXBuffer.Datalength)
		bufferGetFromFront(&TXBuffer,(unsigned char *)&UDR,1);
	else
		TRANSMITTER_PIN = 0;
}

__interrupt void TIMER0_OVF_intr ( void )
{
	TMR0_STOP;
	TIMER_0 = Timer0_preload;
	TMR0_START;
	//  
	if (tmrReceive)
	{
		if (!(--tmrReceive))
		{
			RXMode = RXMODE_HEADER_WAIT; //    
			bufferDumpFromFront(&RXBuffer,1); // ,   
		}
	}
	if (START_SEND)
	{
		if (tmrSend)
		{
			if (!--tmrSend)
			{
				CLEAR_START_SEND_FLAG;
				StartTransmit();
			}
		}
		else
			tmrSend = SEND_TIMEOUT;
	}
}



/* ---  --- */
void init ( void )
{
	TRANSMITTER_DDR = 1;
	TRANSMITTER_PIN = 0;
	usartInit(USART_9600,CHSIZE_8,NO_PARITY,_1_STOP_BITS);
	USART_RX_ENA;
	USART_TX_ENA;
	USART_RXC_INT_ENA;																					//   
	USART_TXC_INT_ENA;
	timer0Init(TMR0_CLK_256,217);																//  ~1 
	TMR0_OVF_INT_ENA;
	// 
	bufferInit(&RXBuffer,malloc(RX_BUFFER_SIZE),RX_BUFFER_SIZE);
	bufferInit(&TXBuffer,malloc(TX_BUFFER_SIZE),TX_BUFFER_SIZE);
	spiMasterInit(SPI_MODE_0,SPI_CLK_128,SPI_MSB_FIRST);					//  SPI
	dataflashInit();																							// 
	Flags = 0;																									//  
	RXMode = RXMODE_HEADER_WAIT;
	LED1_DDR = 1;
	LED1_PIN = 0;
	
	__enable_interrupt();
}

void StartTransmit ( void )
{
	if (TXBuffer.Datalength) //    
	{
		TRANSMITTER_PIN = 1;
	  bufferGetFromFront(&TXBuffer,(unsigned char *)&UDR,1); //   
	}
}

void main ( void )
{
	init();
	while (1)
	{
		if (RECEIVE_COMPLETE) //   
		{
			CLEAR_RECEIVE_COMPLETE_FLAG;
			bufferAddToEnd(&TXBuffer,USART_PACKET_HEADER); //     
			//     
			unsigned int Crc = 0;
			//           
			for (Counter1=0;Counter1<DataLength+PACK_MIN_SIZE-2;Counter1++)
				Crc = Add_to_CRC16(Crc,bufferGetAtIndex(&RXBuffer,Counter1));
			//     
			SET_SEND_ANSWER_FLAG;
			if (Crc16==Crc)
			{
				bufferDumpFromFront(&RXBuffer,3); //      ,    
				unsigned char Command;
				bufferGetFromFront(&RXBuffer,&Command,1); //   
				switch (Command)
				{
					case CMD_CHECK: // 
					{
						DataLength = 1;
						bufferAddDataBlock(&TXBuffer,(unsigned char *)&DataLength,sizeof(DataLength)); //       
						bufferAddToEnd(&TXBuffer,STATUS_OK); //   
					} break;
					case CMD_ERASE_CHIP: //  
					{
						// 
						dataflashChipErase(); // 
						DataLength = 1;
						bufferAddDataBlock(&TXBuffer,(unsigned char *)&DataLength,sizeof(DataLength)); //       
						bufferAddToEnd(&TXBuffer,STATUS_OK); //   
					} break;
					case CMD_READ_MEMORY: //  
					{
						unsigned long ReadAddress;
						bufferGetFromFront(&RXBuffer,(unsigned char *)&ReadAddress,sizeof(ReadAddress)); //    
						DataLength = FLASH_BLOCK_SIZE+1;
						bufferAddDataBlock(&TXBuffer,(unsigned char *)&DataLength,sizeof(DataLength)); //       
						bufferAddToEnd(&TXBuffer,STATUS_OK); //   
						dataflashReadToCBuffer(ReadAddress,FLASH_BLOCK_SIZE,&TXBuffer);
					} break;
					case CMD_WRITE_MEMORY:
					{
						unsigned long WriteAddress;
						bufferGetFromFront(&RXBuffer,(unsigned char *)&WriteAddress,sizeof(WriteAddress)); //   
						dataflashPageUpdateFromCBuffer(WriteAddress,FLASH_BLOCK_SIZE,&RXBuffer);
						DataLength = 1;
						bufferAddDataBlock(&TXBuffer,(unsigned char *)&DataLength,sizeof(DataLength)); //       
						bufferAddToEnd(&TXBuffer,STATUS_OK); //   
					} break;
					default:
					{
						DataLength = 1;
						bufferAddDataBlock(&TXBuffer,(unsigned char *)&DataLength,sizeof(DataLength)); //       
						bufferAddToEnd(&TXBuffer,STATUS_BAD_COMMAND); //   
					} break;
				}
			} //if (Crc16==Crc)
			else
			{
				DataLength = 1;
				bufferAddDataBlock(&TXBuffer,(unsigned char *)&DataLength,sizeof(DataLength)); //       
				bufferAddToEnd(&TXBuffer,STATUS_BAD_CRC); //   
			}
			bufferDumpFromFront(&RXBuffer,2); //     
			RXMode = RXMODE_HEADER_WAIT; //     
			if (SEND_ANSWER) //     
			{
				//  
				Crc16 = 0;
				for (Counter1=0;Counter1<TXBuffer.Datalength;Counter1++)
					Crc16 = Add_to_CRC16(Crc16,bufferGetAtIndex(&TXBuffer,Counter1));
				bufferAddDataBlock(&TXBuffer,(unsigned char *)&Crc16,sizeof(Crc16));
				SET_START_SEND_FLAG;
			}
			else
				bufferFlush(&TXBuffer); //    
		} //if (RECEIVE_COMPLETE)
		else
		{
			if (RXMode==RXMODE_HEADER_WAIT) //  
			{
				if (RXBuffer.Datalength)
				{
					if (bufferGetAtIndex(&RXBuffer,0)==USART_PACKET_HEADER) //  
					{						
						RXMode = RXMODE_LENGTH_WAIT; //      
						tmrReceive = RECEIVE_TIMEOUT; // 
					}
					else
						bufferDumpFromFront(&RXBuffer,1); //  
				}
			}
			if (RXMode==RXMODE_LENGTH_WAIT) //     
			{
				if (RXBuffer.Datalength >= 3)
				{
					DataLength = bufferGetAtIndex(&RXBuffer,1); //  
					DataLength |= (bufferGetAtIndex(&RXBuffer,2)<<8); //  
					if (DataLength+PACK_MIN_SIZE > RX_BUFFER_SIZE) //     ,   
					{
						RXMode = RXMODE_HEADER_WAIT;
						bufferDumpFromFront(&RXBuffer,1); //  
					}
					else
					{
						RXMode = RXMODE_DATA_WAIT;
					}
				}
			}
			if (RXMode==RXMODE_DATA_WAIT) //   
			{
				if (RXBuffer.Datalength >= DataLength + PACK_MIN_SIZE - 2)
				{
					RXMode = RXMODE_CRC_WAIT;
				}
			}
			if (RXMode==RXMODE_CRC_WAIT) //   
			{
				if (RXBuffer.Datalength >= PACK_MIN_SIZE + DataLength) //    
				{
					tmrReceive = 0; // 
					RXMode = RXMODE_NO_PROCESSING; //     
					SET_RECEIVE_COMPLETE_FLAG; //    ,   
					Crc16 = bufferGetAtIndex(&RXBuffer,DataLength+PACK_MIN_SIZE-2); //    
					Crc16 |= (bufferGetAtIndex(&RXBuffer,DataLength+PACK_MIN_SIZE-1)<<8); //    
				}
			}
		} //if (RECEIVE_COMPLETE)...else
	}
}
